import 'dart:async';

import 'package:flutter/services.dart';

class PagseguroSdk {

  static const EVENT_PROCESSING = "EVENT_PROCESSING";
  static const EVENT_PASSWORD_REQUESTED = "EVENT_PROCESSING";
  static const EVENT_PASSWORD_UPDATED = "EVENT_PASSWORD_UPDATED";
  static const EVENT_DECLINED = "EVENT_DECLINED";
  static const EVENT_APPROVED = "EVENT_APPROVED";
  static const EVENT_CANCELLED = "EVENT_CANCELLED";

  static const MethodChannel _channel = const MethodChannel('pagseguro_sdk');
  static const EventChannel _eventsChannel = const EventChannel('pagseguro_events');
  static Stream<dynamic> _eventsStream;

  Future<String> get platformVersion async {
    print('PAGSEGURO: ' + _channel.name);
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<bool> get isSupported async {
    print('PAGSEGURO: ' + _channel.name);
    final bool version = await _channel.invokeMethod('isSupported');
    return version;
  }

  static Future<bool> get isSupportedStatic async {
    print('PAGSEGURO: ' + _channel.name);
    final bool version = await _channel.invokeMethod('isSupported');
    return version;
  }

  static Future<bool> get brand async {
    print('PAGSEGURO: ' + _channel.name);
    final bool version = await _channel.invokeMethod('brand');
    return version;
  }

  Future<bool> initialize({String appVersion, String appName, String activationCode}) async {

    final dynamic result = await _channel.invokeMethod('initialize', {
      'appName': appName,
      'appVersion': appVersion,
      'activationCode': activationCode
    });

    return result;
  }

  Stream<PagseguroEvent> subscribeToEvents() {
    print('PAGSEGURO: wait');
    _eventsStream = _eventsChannel.receiveBroadcastStream();
    print('PAGSEGURO: listen called');
    return _eventsStream.map((value) {
      return PagseguroEvent(
        message: value['message'],
        code: value['code'],
        mask: value['mask'],
        type: value['type'],
        finished: value['finished']
      );
    });
  }

  Future<List<PagseguroInstallmentOption>> getInstallments({num amount}) async {
    return _channel.invokeListMethod('getInstallments', {
      'amount': amount
    }).then((list) {
      return list.map((value) => PagseguroInstallmentOption(
          installmentCount: value['installmentCount'],
          installmentAmount: value['installmentAmount'],
          total: value['installmentAmount'] * value['installmentCount']
      )
      ).toList();
    });
  }

  Future<bool> startPayment({
    num amount,
    int installments,
    String type, Null Function(dynamic event) listener
  }) async {

    print('sdk_wait');
    bool result = await _channel.invokeMethod('startPayment', {
      'amount': amount,
      'installments': installments,
      'type': type,
      'paymentId': '',
      'listener': listener
    });
    print('sdk_result $result');
    return result;
  }

  Future<bool> abortPayment() async {

    await _channel.invokeMethod('abortPayment');

    return true;
  }

}

class PagseguroEvent{
  String message;
  String code;
  String mask;
  String type;
  bool finished;

  PagseguroEvent({this.message, this.code, this.mask, this.type, this.finished});
}

class PagseguroInstallmentOption{
  num installmentCount;
  num installmentAmount;
  num total;

  PagseguroInstallmentOption({
      this.installmentCount,
      this.installmentAmount,
      this.total
  });
}